// Send the required lua files to the client
// Send the required lua files to the client
AddCSLuaFile("cl_init.lua")
AddCSLuaFile("cl_scoreboard.lua")
AddCSLuaFile("cl_scoreboard_playerlist.lua")
AddCSLuaFile("cl_scoreboard_playerrow.lua")
AddCSLuaFile("sh_config.lua")
AddCSLuaFile("sh_init.lua")
AddCSLuaFile("sh_player.lua")


// If there is a mapfile send it to the client (sometimes servers want to change settings for certain maps)
if file.Exists("../gamemodes/prop_hunt/gamemode/maps/"..game.GetMap()..".lua") then
	AddCSLuaFile("maps/"..game.GetMap()..".lua")
end


// Include the required lua files
include("sh_init.lua")


// Server only constants
EXPLOITABLE_DOORS = {
	"func_door",
	"prop_door_rotating", 
	"func_door_rotating"
}
USABLE_PROP_ENTITIES = {
	"prop_physics",
	"prop_physics_multiplayer"
}


// Send the required resources to the client
for _, taunt in pairs(HUNTER_TAUNTS) do resource.AddFile("sound/"..taunt) end
for _, taunt in pairs(PROP_TAUNTS) do resource.AddFile("sound/"..taunt) end


// Prevents the player from being able to suicide
function GM:CanPlayerSuicide(pl)
	if pl:Team() == TEAM_SPECTATOR || pl:GetNWInt("dead") == 1 then
		return false
	end
	
	umsg.Start("PlayerKilledSelf")
	umsg.Entity(pl) 
	umsg.End()
	
	if pl:Team() == TEAM_HUNTERS then
		MsgAll(pl:Name() .. " felt guilty for hurting so many innocent props and committed suicide\n")
	else
		MsgAll(pl:Name() .. " couldn't take the pressure and committed suicide\n")
	end
	
	return true
end


// Called when the player dies
function GM:DoPlayerDeath(pl, attacker, dmginfo)
	pl:RemoveProp()
	
	if round_start_time != 0 then
		pl.death_pos = pl:EyePos()
		pl:SetNWInt("dead", 1)
		pl:AddDeaths(1)
		
		if !table.HasValue(dead_players, pl:UniqueID()) then
			table.insert(dead_players, pl:UniqueID())
		end
	end
	
	if pl:Team() == TEAM_HUNTERS then
		pl:CreateRagdoll()
		pl:UnLock()
	end
end


// Called when the round needs to be ended
function GM:EndRound(winning_team)
	round_start_time = 0
	rounds_complete = rounds_complete + 1
	
	umsg.Start("SetRoundStartTime", RecipientFilter():AddAllPlayers())
	umsg.Long(round_start_time)
	umsg.End()
	
	umsg.Start("ShowWinningTeam", RecipientFilter():AddAllPlayers())
	umsg.Short(winning_team)
	umsg.End()
	
	if winning_team == TEAM_HUNTERS then
		PrintMessage(HUD_PRINTCENTER, "Hunters Win!")
		hunter_wins = hunter_wins + 1
	elseif winning_team == TEAM_PROPS then
		for _, pl in pairs(team.GetPlayers(TEAM_PROPS)) do
			if pl:GetNWInt("dead") == 0 then
				pl:AddFrags(1)
			end
		end
		
		for _, pl in pairs(team.GetPlayers(TEAM_HUNTERS)) do
			pl:StripWeapons()
		end
		
		PrintMessage(HUD_PRINTCENTER, "Props Win!")
		prop_wins = prop_wins + 1
	end
	
	umsg.Start("UpdateWinCount", RecipientFilter():AddAllPlayers())
	umsg.Long(hunter_wins)
	umsg.Long(prop_wins)
	umsg.End()
	
	// Unlock/unblind hunters
	for _, pl in pairs(team.GetPlayers(TEAM_HUNTERS)) do
		pl:Blind(false)
		pl:UnLock()
	end

	// If this is the last round for the map 
	if rounds_complete == ROUNDS_PER_MAP then
		switching_maps = 1
		timer.Simple(5, self.LockAndShowScoreboard, self)
		timer.Simple(10, game.LoadNextMap)
	else
		round_end_time = CurTime()
	end
end


// Called when an entity takes damage
function GM:EntityTakeDamage(ent, inflictor, attacker, amount)
	if round_start_time > 0 && ent && ent:GetClass() != "ph_prop" && !ent:IsPlayer() && attacker && attacker:IsPlayer() && attacker:Team() == TEAM_HUNTERS && attacker:Alive() then
		attacker:SetHealth(attacker:Health() - HUNTER_FIRE_PENALTY)
		if attacker:Health() <= 0 then
			MsgAll(attacker:Name() .. " felt guilty for hurting so many innocent props and committed suicide\n")
			attacker:Kill()
		end
	end
end


// Gets hunter/prop spawns
function GM:GetSpawnPoints()
	hunter_spawns = {}
	table.Add(hunter_spawns, ents.FindByClass("info_player_counterterrorist"))
	table.Add(hunter_spawns, ents.FindByClass("info_player_combine"))
	table.Add(hunter_spawns, ents.FindByClass("info_player_deathmatch"))
	table.Add(hunter_spawns, ents.FindByClass("info_player_axis"))
	
	prop_spawns = {}
	table.Add(prop_spawns, ents.FindByClass("info_player_terrorist"))
	table.Add(prop_spawns, ents.FindByClass("info_player_rebel"))
	table.Add(prop_spawns, ents.FindByClass("info_player_deathmatch"))
	table.Add(prop_spawns, ents.FindByClass("info_player_allies"))
end


// Called when the gamemode is initialized
function GM:Initialize()
	round_start_time = 0
	round_end_time = 0
	rounds_complete = 0
	
	hunter_wins = 0
	prop_wins = 0
	
	dead_players = {}
	
	game.ConsoleCommand("mapcyclefile "..MAPCYCLE_FILE.."\n")
	game.ConsoleCommand("mp_flashlight "..ALLOW_FLASHLIGHT.."\n")
	game.ConsoleCommand("mp_falldamage 0\n")
end


// Called after all entities have been spawned
function GM:InitPostEntity()
	self:GetSpawnPoints()
	self:RemoveWeaponsAndItems()
end


// Locks all players && shows the scoreboard
function GM:LockAndShowScoreboard()
	for _, pl in pairs(player.GetAll()) do
		pl:Lock()
		pl:ConCommand("+score")
	end
end


// Called when a player changes teams
function GM:OnPlayerChangedTeam(pl, old_team, new_team)
	if new_team == TEAM_SPECTATOR then
		local pos = pl:EyePos()
		pl:Spawn()
		pl:SetPos(pos)
		pl:SetNWInt("dead", 0)
		PrintMessage(HUD_PRINTTALK, Format("%s is now spectating.", pl:Nick()))
	else
		pl:Spawn()
		PrintMessage(HUD_PRINTTALK, Format("%s joined %s.", pl:Nick(), team.GetName(new_team)))
	end
end


// Checks to see if a player can join a team or not
function GM:PlayerCanJoinTeam(pl, team_id)
	if team_id == TEAM_SPECTATOR then
		return true
	end
	
	if round_start_time > 0 && pl.switched_this_round then
		pl:ChatPrint("You must wait until next round before switching teams again.")
		return false
	end
	
	local remove_one = 1
	if pl:Team() == TEAM_SPECTATOR then
		remove_one = 0
	end
	
	if (team_id == TEAM_HUNTERS && team.NumPlayers(TEAM_HUNTERS) + 1 - MAX_UNBALANCE > math.Clamp(team.NumPlayers(TEAM_PROPS) - remove_one, 0, 128)) || (team_id == TEAM_PROPS && team.NumPlayers(TEAM_PROPS) + 1 - MAX_UNBALANCE > math.Clamp(team.NumPlayers(TEAM_HUNTERS) - remove_one, 0, 128)) then
		pl:ChatPrint("That team is full!")
		return false
	end
	
	return true
end


// Called when player tries to pickup a weapon
function GM:PlayerCanPickupWeapon(pl, ent)
 	if pl:Team() != TEAM_HUNTERS then
		return false
	end
	
	return true
end


// Called when a player leaves
function GM:PlayerDisconnected(pl)
	pl:RemoveProp()
end


// Does the killicons && such
function GM:PlayerDeath(pl, inflictor, attacker)
	pl.NextSpawnTime = CurTime() + 5
end


// Called right before the player's first spawn
function GM:PlayerInitialSpawn(pl)
	pl:SetNWInt("dead", 0)
	pl:SetTeam(TEAM_SPECTATOR)
	pl:ConCommand("gm_showteam")
	pl.switched_this_round = false
	
	umsg.Start("FirstTimeHelp", pl)
	umsg.End()
	
	umsg.Start("UpdateWinCount", pl)
	umsg.Long(hunter_wins)
	umsg.Long(prop_wins)
	umsg.End()
end


// Called when a player joins a team
function GM:PlayerJoinTeam(pl, team_id)
	pl:SendLua("GAMEMODE:HideTeam()")

	local old_team = pl:Team()
	
	if pl:Alive() then
		pl:KillSilent()
	end

	pl:SetTeam(team_id)
	pl.switched_this_round = true
	
	self:OnPlayerChangedTeam(pl, old_team, team_id)
end


// Called when player needs guns
function GM:PlayerLoadout(pl)
	pl:GiveAmmo(64, "Buckshot")
	pl:GiveAmmo(255, "SMG1")
	
	pl:Give("weapon_crowbar")
	pl:Give("weapon_shotgun")
	pl:Give("weapon_smg1")
	pl:Give("item_ar2_grenade")
	
	local cl_defaultweapon = pl:GetInfo("cl_defaultweapon") 
 	 
 	if pl:HasWeapon(cl_defaultweapon) then 
 		pl:SelectWeapon(cl_defaultweapon)
 	end 
end


// Called when player needs a model
function GM:PlayerSetModel(pl)
	local player_model = "models/Gibs/Antlion_gib_small_3.mdl"
	if pl:Team() == TEAM_HUNTERS then player_model = "models/player/combine_super_soldier.mdl" end
	
	util.PrecacheModel(player_model)
	pl:SetModel(player_model)
end


// Called when the players spawns
function GM:PlayerSpawn(pl)
	if pl.switching_props then
		pl:SetPos(pl.prev_pos)
		pl:SnapEyeAngles(pl.prev_angles:Angle())
		pl.switching_props = false
		return
	end
	
	pl:Blind(false)
	pl:RemoveProp()
	pl:SetColor(255, 255, 255, 255)
	pl:UnLock()
	pl:ResetHull()
	pl.last_taunt_time = 0
	
	umsg.Start("ResetHull", pl)
	umsg.End()
	
	if pl:Team() == TEAM_SPECTATOR || pl:GetNWInt("dead") == 1 then
		pl:Spectate(OBS_MODE_ROAMING)
		
		if pl.death_pos then
			pl:SetPos(pl.death_pos)
		end
		
		return
	end
	
	pl:UnSpectate()
	
	umsg.Start("SetRoundStartTime", pl)
	umsg.Long(round_start_time)
	umsg.End()
	
	if pl:Team() == TEAM_HUNTERS then
		if round_start_time > 0 then
			local unlock_time = math.Clamp(HUNTER_BLINDLOCK_TIME - (CurTime() - round_start_time), 0, HUNTER_BLINDLOCK_TIME)
			
			if unlock_time > 2 then
				pl:Blind(true)
				timer.Simple(unlock_time, pl.Blind, pl, false)
				
				timer.Simple(2, pl.Lock, pl)
				timer.Simple(unlock_time, pl.UnLock, pl)
			end
		end
		
		self:SetPlayerSpeed(pl, 230, 230)
		self:PlayerLoadout(pl)
	elseif pl:Team() == TEAM_PROPS then
		pl:SetColor(255, 255, 255, 0)
		
		pl.ph_prop = ents.Create("ph_prop")
		pl.ph_prop:SetPos(pl:GetPos())
		pl.ph_prop:SetAngles(pl:GetAngles())
		pl.ph_prop:Spawn()
		pl.ph_prop:SetSolid(SOLID_BBOX)
		pl.ph_prop:SetParent(pl)
		pl.ph_prop:SetOwner(pl)
		
		pl.ph_prop.max_health = 100
		
		self:SetPlayerSpeed(pl, 250, 250)
	end
	
	self:PlayerSetModel(pl)
	
	pl:SetCollisionGroup(COLLISION_GROUP_PASSABLE_DOOR)
	
	if (pl:Team() == TEAM_HUNTERS || pl:Team() == TEAM_PROPS) && table.HasValue(dead_players, pl:UniqueID()) then
		pl:PrintMessage(HUD_PRINTTALK, "You may not respawn until the next round. Nice try though.")
		pl:Kill()
	end
end


// Can the flashlight be on?
function GM:PlayerSwitchFlashlight(pl, on)
	if pl:Team() == TEAM_HUNTERS && GetConVarNumber("mp_flashlight") == 1 then
		return true
	end
	
	return false
end


// Called when a player tries to use an object
function GM:PlayerUse(pl, ent)
	if pl:GetNWInt("dead") == 1 || pl:Team() == TEAM_SPECTATOR then return false end
	
	if pl:Team() == TEAM_PROPS && pl:IsOnGround() && !pl:Crouching() && table.HasValue(USABLE_PROP_ENTITIES, ent:GetClass()) && ent:GetModel() then
		if table.HasValue(BANNED_PROP_MODELS, ent:GetModel()) then
			pl:ChatPrint("That prop has been banned by the server.")
		elseif ent:GetPhysicsObject():IsValid() && pl.ph_prop:GetModel() != ent:GetModel() then
			local ent_health = math.Clamp(ent:GetPhysicsObject():GetVolume() / 250, 1, 200)
			local new_health = math.Clamp((pl.ph_prop.health / pl.ph_prop.max_health) * ent_health, 1, 200)
			local per = pl.ph_prop.health / pl.ph_prop.max_health
			pl.ph_prop.health = new_health
			
			pl.ph_prop.max_health = ent_health
			pl.ph_prop:SetModel(ent:GetModel())
			pl.ph_prop:SetSkin(ent:GetSkin())
			pl.ph_prop:SetSolid(SOLID_BSP)
			pl.ph_prop:SetPos(pl:GetPos() - Vector(0, 0, ent:OBBMins().z))
			pl.ph_prop:SetAngles(pl:GetAngles())
			
			local hullxymax = math.Round(math.Max(ent:OBBMaxs().x, ent:OBBMaxs().y))
			local hullxymin = hullxymax * -1
			local hullz = math.Round(ent:OBBMaxs().z)
			
			pl:SetHull(Vector(hullxymin, hullxymin, 0), Vector(hullxymax, hullxymax, hullz))
			pl:SetHullDuck(Vector(hullxymin, hullxymin, 0), Vector(hullxymax, hullxymax, hullz))
			
			umsg.Start("SetHull", pl)
				umsg.Long(hullxymax)
				umsg.Long(hullz)
			umsg.End()
			
			pl.switching_props = true
			pl.prev_pos = pl:GetPos()
			pl.prev_angles = pl:GetAimVector()
			pl:Spawn()
			pl:SetHealth(new_health)
		end
	end
	
	// Prevent the door exploit
	if table.HasValue(EXPLOITABLE_DOORS, ent:GetClass()) && pl.last_door_time && pl.last_door_time + 1 > CurTime() then
		return false
	end
	
	pl.last_door_time = CurTime()
	return true
end


// Removes all weapons on a map
function GM:RemoveWeaponsAndItems()
	for _, wep in pairs(ents.FindByClass("weapon_*")) do
		wep:Remove()
	end
	
	for _, item in pairs(ents.FindByClass("item_*")) do
		item:Remove()
	end
end


// Called when a player presses the help key
function GM:ShowHelp(pl)
	umsg.Start("OpenHelpMenu", pl)
	umsg.End()
end


// Taunts
function GM:ShowSpare1(pl)
	if (pl:Team() == TEAM_HUNTERS || pl:Team() == TEAM_PROPS) && pl:GetNWInt("dead") == 0 && #PROP_TAUNTS > 1 && pl.last_taunt_time + TAUNT_DELAY <= CurTime() then
		repeat
			if pl:Team() == TEAM_HUNTERS then
				rand_taunt = table.Random(HUNTER_TAUNTS)
			else
				rand_taunt = table.Random(PROP_TAUNTS)
			end
		until rand_taunt != pl.last_taunt
		
		pl.last_taunt_time = CurTime()
		pl.last_taunt = rand_taunt
		
		pl:EmitSound(rand_taunt, 100)
	end	
end


// Called when the round needs to be started
function GM:StartRound()
	round_start_time = CurTime()
	dead_players = {}
	
	// Cleanup the map
	game.CleanUpMap()
	
	// For each player...
	for _, pl in pairs(player.GetAll()) do
		if pl:Team() == TEAM_PROPS || pl:Team() == TEAM_HUNTERS then
			pl:KillSilent()
			
			if rounds_complete != 0 && SWAP_TEAMS_EVERY_ROUND == 1 then
				if pl:Team() == TEAM_PROPS then
					pl:SetTeam(TEAM_HUNTERS)
				else
					pl:SetTeam(TEAM_PROPS)
				end
				
				pl:ChatPrint("Teams have been swapped!")
			end
			
			pl:SetNWInt("dead", 0)
			pl:Spawn()
		end
		
		pl.switched_this_round = false
 	end
end 


// Called every frame
function GM:Think()
	if team.NumPlayers(TEAM_PROPS) - MAX_UNBALANCE > team.NumPlayers(TEAM_HUNTERS) then
		for _, pl in pairs(team.GetPlayers(TEAM_PROPS)) do
			if pl:GetNWInt("dead") == 1 then
				pl:SetTeam(TEAM_HUNTERS)
				pl:PrintMessage(HUD_PRINTTALK, "You have been moved to the HUNTER team for team balance.")
				break
			end
		end
	end
	
	if team.NumPlayers(TEAM_HUNTERS) - MAX_UNBALANCE > team.NumPlayers(TEAM_PROPS) then
		for _, pl in pairs(team.GetPlayers(TEAM_HUNTERS)) do
			if pl:GetNWInt("dead") == 1 then
				pl:SetTeam(TEAM_PROPS)
				pl:PrintMessage(HUD_PRINTTALK, "You have been moved to the PROP team for team balance.")
				break
			end
		end
	end
	
	if round_start_time == 0 && rounds_complete != ROUNDS_PER_MAP && team.NumPlayers(TEAM_PROPS) > 0 && team.NumPlayers(TEAM_HUNTERS) > 0 && round_end_time + 5 < CurTime() then
		self:StartRound()
	elseif round_start_time > 0 then
		if round_start_time + ROUND_TIME <= CurTime() then
			self:EndRound(TEAM_PROPS)
		else
			local hunters_alive = 0
			local props_alive = 0
			
			for _, pl in pairs(player.GetAll()) do
				if pl:GetNWInt("dead") == 0 then
					if pl:Team() == TEAM_HUNTERS then
						hunters_alive = 1
					elseif pl:Team() == TEAM_PROPS then
						props_alive = 1
					end
				end
			end
			
			if hunters_alive == 0 then
				self:EndRound(TEAM_PROPS)
			elseif props_alive == 0 then
				self:EndRound(TEAM_HUNTERS)
			end
		end
	end
end